@@ -1,15 +1,13 @@
 package Huawei::Healthchecker;
 
 #版本信息
-our $VERSION = "0.03";
+our $VERSION = "0.05";
 
-use 5.012;
+use strict;
 use warnings;
-use Data::Dumper;
-
-use Carp;
 use Net::Compare;
 
+#引用导出函数
 use Exporter;
 
 #继承parent相关变量
@@ -45,14 +43,83 @@ my $hardware = $attr->{"hardware"};
 #固化关键指标巡检参数为数组（哈希无序）
 my $status = $attr->{"status"};
 
+#-------------------------------------------------------------------------
+# 网络设备- 关键指标巡检阈值
+#-------------------------------------------------------------------------
+#CPU告警阈值
+my $cpu_threshold = 60;
+
+#内存告警阈值
+my $memory_threshold = 80;
+
+#CPU历史MAX告警阈值
+my $cpu_max_threshold = 85;
+
+#磁盘告警阈值
+my $disk_threshold = 75;
+
 #-------------------------------------------------------------------------
 # 网络设备-华为设备巡检入口函数
 #-------------------------------------------------------------------------
 sub health_check {
+
+    #接收待巡检配置文件
     my $config = shift;
-    my $rev    = status_check($config);
-    my $score  = calculate_score($rev);
-    return [ $rev, { "score" => $score } ];
+
+    #解析巡检输出
+    my $rev = status_check($config);
+
+    #初始化变量-设备巡检日志
+    my @comment;
+
+    #获取设备版本信息
+    #my $verison = $rev->{"verison"};
+
+    #初始化设备硬件状态巡检结果-数组
+    my @hardware_ret;
+    push @hardware_ret, $rev->{"hardware"}{"device_module"}{"health"};
+    push @hardware_ret, $rev->{"hardware"}{"device_power"}{"health"};
+    push @hardware_ret, $rev->{"hardware"}{"device_disk"}{"health"};
+    push @hardware_ret, $rev->{"hardware"}{"device_fan"}{"health"};
+    push @hardware_ret, $rev->{"hardware"}{"device_temperature"}{"health"};
+    push @hardware_ret, $rev->{"hardware"}{"device_alarm"}{"health"};
+
+    #初始化设备关键指标巡检结果-数组
+    my @status_ret;
+    push @status_ret, $rev->{"status"}{"device_cpu"}{"health"};
+    push @status_ret, $rev->{"status"}{"device_memory"}{"health"};
+    push @status_ret, $rev->{"status"}{"device_ntp"}{"health"};
+    push @status_ret, $rev->{"status"}{"device_vrrp"}{"health"};
+    push @status_ret, $rev->{"status"}{"device_eth_trunk"}{"health"};
+    push @status_ret, $rev->{"status"}{"device_err_down"}{"health"};
+
+    #遍历 $rev->{"hardware"} 各属性哈希值
+    foreach ( keys %{ $rev->{"hardware"} } ) {
+        my $ret = $rev->{"hardware"}->{$_};
+        if ( defined $ret->{"log"} ) {
+            push @comment, $ret->{"log"};
+        }
+    }
+
+    #遍历 $rev->{"status"} 各属性哈希值
+    foreach ( keys %{ $rev->{"status"} } ) {
+        my $ret = $rev->{"status"}->{$_};
+        if ( defined $ret->{"log"} ) {
+            push @comment, $ret->{"log"};
+        }
+    }
+
+    #将设备描述数组转换未字符串
+    my $comment = join( "\n", @comment ) if @comment;
+
+    #计算设备巡检健康度
+    my $score = calculate_score($rev);
+
+    #封装计算结果到匿名数组中，方便EXCEL读写
+    my $result = [ \@hardware_ret, \@status_ret, $score, $comment ];
+
+    #返回健康检查结果
+    return ( $ret, $result );
 }
 
 #-------------------------------------------------------------------------
@@ -69,12 +136,12 @@ sub status_check {
     #-------------------------------------------------------------------------
     # 网络设备-硬件运行状态巡检命令行输出
     #-------------------------------------------------------------------------
-    my $device_version = catch_cmd( $hardware->{"device_version"}, $config );
-    my $device_module  = catch_cmd( $hardware->{"module_status"},  $config );
-    my $device_power   = catch_cmd( $hardware->{"power_status"},   $config );
-    my $device_disk    = catch_cmd( $hardware->{"disk_status"},    $config );
-    my $device_fan     = catch_cmd( $hardware->{"fan_status"},     $config );
-    my $device_alarm   = catch_cmd( $hardware->{"alarm_status"},   $config );
+    #my $device_version = catch_cmd( $hardware->{"device_version"}, $config );
+    my $device_module = catch_cmd( $hardware->{"module_status"}, $config );
+    my $device_power  = catch_cmd( $hardware->{"power_status"},  $config );
+    my $device_disk   = catch_cmd( $hardware->{"disk_status"},   $config );
+    my $device_fan    = catch_cmd( $hardware->{"fan_status"},    $config );
+    my $device_alarm  = catch_cmd( $hardware->{"alarm_status"},  $config );
     my $device_temperature
         = catch_cmd( $hardware->{"temperature_status"}, $config );
 
@@ -92,7 +159,7 @@ sub status_check {
     #-------------------------------------------------------------------------
     # 网络设备-硬件运行状态巡检判断
     #-------------------------------------------------------------------------
-    my $device_info        = device_version($device_version);
+    #my $device_info        = device_version($device_version);
     my $module_status      = module_status($device_module);
     my $power_status       = power_status($device_power);
     my $disk_status        = disk_status($device_disk);
@@ -111,9 +178,13 @@ sub status_check {
     my $err_down_status  = err_down_status($device_err_down);
 
     #-------------------------------------------------------------------------
-    # 网络设备-硬件运行状态巡检结果 --共计7项
+    # 网络设备-设备版本型号信息 --检查项1 共计1项
+    #-------------------------------------------------------------------------
+    #$result->{"version"}     = $device_info;
+
+    #-------------------------------------------------------------------------
+    # 网络设备-硬件运行状态巡检结果 --检查项2 共计6项
     #-------------------------------------------------------------------------
-    $result->{"hardware"}{"device_version"}     = $device_info;
     $result->{"hardware"}{"device_module"}      = $module_status;
     $result->{"hardware"}{"device_power"}       = $power_status;
     $result->{"hardware"}{"device_fan"}         = $fan_status;
@@ -122,7 +193,7 @@ sub status_check {
     $result->{"hardware"}{"device_temperature"} = $temperature_status;
 
     #-------------------------------------------------------------------------
-    # 网络设备-关键运行指标巡检结果 --共计6项
+    # 网络设备-关键运行指标巡检结果 --检查项3 共计6项
     #-------------------------------------------------------------------------
     $result->{"status"}{"device_cpu"}       = $cpu_status;
     $result->{"status"}{"device_memory"}    = $memory_status;
@@ -214,46 +285,27 @@ sub device_version {
     #如果没匹配到巡检输出直接返回
     return unless $rev;
 
-    #模块巡检标量
-    my $result;
-
-    #初始化巡检结果告警项
-    $result->{"alarm"} = undef;
-
-    #重组巡检结果拼接为字符串
-    my $origin = join( "\n", @{$rev} );
-    $result->{"origin"} = $origin;
-
-    #快速检查是否有未注册的模块
-    my $ret = ( grep {/Unregistered/} @{$rev} ) ? 1 : 0;
-
-    #设备健康检查
-    $result->{"health"} = "pass";
-    return $result unless $ret;
+    #设备版本标量
+    my $version;
 
     #遍历巡检结果以捕捉异常模块信息
     foreach ( @{$rev} ) {
-        if (/(?<alarm>Unregistered)/) {
+        if (/Version\s(.*?)\s(.*)/) {
 
             #将巡检结果命令行分割为字符串数组
-            my @info = split( /\s+/, $_ );
+            my $version = $2;
 
-            #母板默认显示为-
-            $info[1] = "main_board" if @info && $info[1] eq '-';
+            #将非必要字符串（）处理掉
+            $version =~ s/\(//;
+            $version =~ s/\)//;
 
-            #写入异常状态
-            my $alarm = lc( $+{alarm} );
-            $result->{"log"}   = $info[1] . " is $alarm";
-            $result->{"alarm"} = "critic";
-
-            #监控检查状态
-            $result->{"health"} = "fail";
+            #捕捉到关键字即跳出
             last;
         }
     }
 
     #返回巡检结果
-    return $result;
+    return $version;
 }
 
 #-------------------------------------------------------------------------
@@ -271,7 +323,9 @@ sub module_status {
     my $result;
 
     #初始化巡检结果告警项
-    $result->{"alarm"} = undef;
+    $result->{"alarm"}  = undef;
+    $result->{"health"} = "pass";
+    $result->{"log"}    = undef;
 
     #重组巡检结果拼接为字符串
     my $origin = join( "\n", @{$rev} );
@@ -281,7 +335,6 @@ sub module_status {
     my $ret = ( grep {/Unregistered/} @{$rev} ) ? 1 : 0;
 
     #设备健康检查
-    $result->{"health"} = "pass";
     return $result unless $ret;
 
     #遍历巡检结果以捕捉异常模块信息
@@ -296,7 +349,8 @@ sub module_status {
 
             #写入异常状态
             my $alarm = lc( $+{alarm} );
-            $result->{"log"}   = $info[1] . " is $alarm";
+            $result->{"log"}
+                = "设备板卡告警： " . $info[1] . " 未加载";
             $result->{"alarm"} = "critic";
 
             #监控检查状态
@@ -324,7 +378,9 @@ sub power_status {
     my $result;
 
     #初始化巡检结果告警项
-    $result->{"alarm"} = undef;
+    $result->{"alarm"}  = undef;
+    $result->{"health"} = "pass";
+    $result->{"log"}    = undef;
 
     #重组巡检结果拼接为字符串
     my $origin = join( "\n", @{$rev} );
@@ -334,7 +390,6 @@ sub power_status {
     my $ret = ( grep {/NotSupply/i} @{$rev} ) ? 1 : 0;
 
     #设备健康检查
-    $result->{"health"} = "pass";
     return $result unless $ret;
 
     #遍历巡检结果以捕捉异常模块信息
@@ -346,7 +401,8 @@ sub power_status {
 
             #写入异常状态
             my $alarm = lc( $+{alarm} );
-            $result->{"log"}   = $info[1] . " is $alarm";
+            $result->{"log"}
+                = "电源模块告警： " . $info[1] . " 未供电";
             $result->{"alarm"} = "critic";
 
             #监控检查状态
@@ -374,7 +430,9 @@ sub fan_status {
     my $result;
 
     #初始化巡检结果告警项
-    $result->{"alarm"} = undef;
+    $result->{"alarm"}  = undef;
+    $result->{"health"} = "pass";
+    $result->{"log"}    = undef;
 
     #重组巡检结果拼接为字符串
     my $origin = join( "\n", @{$rev} );
@@ -384,7 +442,6 @@ sub fan_status {
     my $ret = ( grep {/Abnormal/i} @{$rev} ) ? 1 : 0;
 
     #设备健康检查
-    $result->{"health"} = "pass";
     return $result unless $ret;
 
     #遍历巡检结果以捕捉异常模块信息
@@ -396,7 +453,8 @@ sub fan_status {
 
             #写入异常状态
             my $alarm = lc( $+{alarm} );
-            $result->{"log"}   = $info[1] . " is $alarm";
+            $result->{"log"}
+                = "设备风扇： " . $info[1] . " 未正常工作";
             $result->{"alarm"} = "alert";
 
             #监控检查状态
@@ -424,9 +482,9 @@ sub alarm_status {
     my $result;
 
     #初始化巡检结果告警项
-    $result->{"alarm"}{"info"}   = undef;
-    $result->{"alarm"}{"alert"}  = undef;
-    $result->{"alarm"}{"critic"} = undef;
+    $result->{"alarm"}  = undef;
+    $result->{"health"} = "pass";
+    $result->{"log"}    = undef;
 
     #重组巡检结果拼接为字符串
     my $origin = join( "\n", @{$rev} );
@@ -436,7 +494,6 @@ sub alarm_status {
     my $ret = ( grep {/(critic|alert)/i} @{$rev} ) ? 1 : 0;
 
     #设备健康检查
-    $result->{"health"} = "pass";
     return $result unless $ret;
 
  #遍历巡检结果以捕捉异常模块信息,仅关注最新的告警信息
@@ -476,7 +533,9 @@ sub temperature_status {
     my $result;
 
     #初始化巡检结果告警项
-    $result->{"alarm"} = undef;
+    $result->{"alarm"}  = undef;
+    $result->{"health"} = "pass";
+    $result->{"log"}    = undef;
 
     #重组巡检结果拼接为字符串
     my $origin = join( "\n", @{$rev} );
@@ -486,7 +545,6 @@ sub temperature_status {
     my $ret = ( grep {/(Abnormal)/i} @{$rev} ) ? 1 : 0;
 
     #设备健康检查
-    $result->{"health"} = "pass";
     return $result unless $ret;
 
  #遍历巡检结果以捕捉异常模块信息,仅关注最新的告警信息
@@ -498,7 +556,7 @@ sub temperature_status {
 
             #写入异常状态
             my $alarm = lc( $+{alarm} );
-            $result->{"log"}   = "temperature is high " . "->$info[6]";
+            $result->{"log"}   = "设备温度告警：" . "->$info[6]";
             $result->{"alarm"} = "info";
 
             #监控检查状态
@@ -526,18 +584,16 @@ sub disk_status {
     my $result;
 
     #初始化巡检结果告警项
-    $result->{"alarm"}{"info"}   = undef;
-    $result->{"alarm"}{"alert"}  = undef;
-    $result->{"alarm"}{"critic"} = undef;
+    $result->{"alarm"}  = undef;
+    $result->{"health"} = "pass";
+    $result->{"log"}    = undef;
 
     #重组巡检结果拼接为字符串
     my $origin = join( "\n", @{$rev} );
     $result->{"origin"} = $origin;
 
-    #设置缺省检查结果为PASS
-    $result->{"health"} = "pass";
-
-    my $storage_status = 0;
+    #初始化硬盘巡检状态
+    my $status = 0;
     my $storage;
 
  #遍历巡检结果以捕捉异常模块信息,仅关注最新的告警信息
@@ -545,22 +601,22 @@ sub disk_status {
 
         #如果命中磁盘关键字将状态指为1
         if (/System Disk Usage Information/i) {
-            $storage_status = 1;
+            $status = 1;
             push @{$storage}, $_;
         }
 
         #如果命中了存储关键字继续
-        elsif ($storage_status) {
+        elsif ($status) {
 
             #写入后续巡检结果
             push @{$storage}, $_;
         }
 
         #检查是否命中主机名
-        elsif (/^\<.*\>/i) {
+        elsif (/^(\<.*\>|\[.*\])/i) {
 
             #将巡检输出置0，结束命令行捕捉
-            $storage_status = 0;
+            $status = 0;
             last;
         }
     }
@@ -569,11 +625,12 @@ sub disk_status {
         if (/\s+(\d+)\%\s+/i) {
 
             #写入异常状态
-            $result->{"log"}   = "Storage usage is high: " . $1 if $1 > 65;
+            $result->{"log"} = "磁盘空间利用率告警：" . "$1\%"
+                if ( $1 > $disk_threshold );
             $result->{"alarm"} = "info";
 
             #监控检查状态
-            $result->{"health"} = "fail";
+            $result->{"health"} = "fail" if ( $1 > $disk_threshold );
         }
     }
 
@@ -596,14 +653,15 @@ sub cpu_status {
     my $result;
 
     #初始化巡检结果告警项
-    $result->{"alarm"} = undef;
+    $result->{"alarm"}  = undef;
+    $result->{"health"} = "pass";
+    $result->{"log"}    = undef;
 
     #重组巡检结果拼接为字符串
     my $origin = join( "\n", @{$rev} );
     $result->{"origin"} = $origin;
 
     #设置缺省检查结果为PASS
-    $result->{"health"} = "pass";
 
  #遍历巡检结果以捕捉异常模块信息,仅关注最新的告警信息
     foreach ( @{$rev} ) {
@@ -622,8 +680,9 @@ sub cpu_status {
             $usage =~ s/\%//;
 
             #写入异常状态
-            $result->{"log"} = "cpu usage is high: " . "$info[1]\%"
-                if $usage >= 60;
+            $result->{"log"}
+                = "CPU利用率超出当前阈值： " . "$info[1]"
+                if $usage >= $cpu_threshold;
 
             #设置越线惩罚机制
             if ( 60 <= $usage && $usage < 70 ) {
@@ -637,7 +696,7 @@ sub cpu_status {
             }
 
             #监控检查状态
-            $result->{"health"} = "fail" if ( $usage >= 60 );
+            $result->{"health"} = "fail" if ( $usage >= $cpu_threshold );
         }
 
         #------------------------------------------------------
@@ -658,13 +717,14 @@ sub cpu_status {
             $max_usage =~ s/\%//;
 
             #写入异常状态
-            $result->{"log"} = "cpu history usage is high: " . "$info[1]\%"
-                if $max_usage > 85;
+            $result->{"log"}
+                = "CPU历史最大利用率超出阈值： " . "$info[1]"
+                if $max_usage > $cpu_max_threshold;
 
-            $result->{"alarm"} = "critic" if $max_usage > 85;
+            $result->{"alarm"} = "critic" if $max_usage > $cpu_max_threshold;
 
             #监控检查状态
-            $result->{"health"} = "fail" if $max_usage > 85;
+            $result->{"health"} = "fail" if $max_usage > $cpu_max_threshold;
             last;
         }
     }
@@ -688,14 +748,15 @@ sub memory_status {
     my $result;
 
     #初始化巡检结果告警项
-    $result->{"alarm"} = undef;
+    $result->{"alarm"}  = undef;
+    $result->{"health"} = "pass";
+    $result->{"log"}    = undef;
 
     #重组巡检结果拼接为字符串
     my $origin = join( "\n", @{$rev} );
     $result->{"origin"} = $origin;
 
     #设置缺省检查结果为PASS
-    $result->{"health"} = "pass";
 
  #遍历巡检结果以捕捉异常模块信息,仅关注最新的告警信息
     foreach ( @{$rev} ) {
@@ -714,8 +775,9 @@ sub memory_status {
             $usage =~ s/\%//;
 
             #写入异常状态
-            $result->{"log"} = "memory usage is high: " . $usage
-                if ( $usage > 60 );
+            $result->{"log"}
+                = "内存利用率超出当前阈值： " . $info[1]
+                if ( $usage > $memory_threshold );
 
             #设置越线惩罚机制
             if ( 60 <= $usage && $usage < 70 ) {
@@ -729,7 +791,7 @@ sub memory_status {
             }
 
             #监控检查状态
-            $result->{"health"} = "fail" if ( $usage >= 70 );
+            $result->{"health"} = "fail" if ( $usage >= $memory_threshold );
         }
     }
 
@@ -752,12 +814,13 @@ sub ntp_status {
     my $result;
 
     #初始化巡检结果告警项
-    $result->{"alarm"} = undef;
+    $result->{"alarm"}  = undef;
+    $result->{"health"} = "pass";
+    $result->{"log"}    = undef;
 
     #重组巡检结果拼接为字符串
     my $origin = join( "\n", @{$rev} );
     $result->{"origin"} = $origin;
-    $result->{"health"} = "pass";
 
  #遍历巡检结果以捕捉异常模块信息,仅关注最新的告警信息
     foreach ( @{$rev} ) {
@@ -773,7 +836,7 @@ sub ntp_status {
             $status =~ s/(^\s+|\s+$)//;
 
             #写入异常状态
-            $result->{"log"} = "ntp not synchronized"
+            $result->{"log"} = "NTP时钟源未同步"
                 if ( $status =~ /not/i );
             $result->{"alarm"} = "info";
 
@@ -801,7 +864,9 @@ sub vrrp_status {
     my $result;
 
     #初始化巡检结果告警项
-    $result->{"alarm"} = undef;
+    $result->{"alarm"}  = undef;
+    $result->{"health"} = "pass";
+    $result->{"log"}    = undef;
 
     #重组巡检结果拼接为字符串
     my $origin = join( "\n", @{$rev} );
@@ -811,12 +876,11 @@ sub vrrp_status {
     my $ret = ( grep {/Info(.*?)exist/i} @{$rev} ) ? 1 : 0;
 
     #巡检结果命令行输出不存在则跳过
-    $result->{"health"} = "pass";
     return $result if $ret;
 
  #遍历巡检结果以捕捉异常模块信息,仅关注最新的告警信息
     foreach ( @{$rev} ) {
-        if (/init/i) {
+        if (/Initialize/i) {
 
             #将巡检结果命令行分割为字符串数组
             my @info = map {s/(^\s+|\s+$)//r} split( /\:/, $_ );
@@ -828,7 +892,7 @@ sub vrrp_status {
             $status =~ s/(^\s+|\s+$)//;
 
             #写入异常状态
-            $result->{"info"} = "vrrp not synchronized"
+            $result->{"info"} = "VRRP处于Initialize状态"
                 if ( $status =~ /not/i );
 
             #设置告警等级
@@ -858,7 +922,9 @@ sub eth_trunk_status {
     my $result;
 
     #初始化巡检结果告警项
-    $result->{"alarm"} = undef;
+    $result->{"alarm"}  = undef;
+    $result->{"health"} = "pass";
+    $result->{"log"}    = undef;
 
     #重组巡检结果拼接为字符串
     my $origin = join( "\n", @{$rev} );
@@ -868,12 +934,11 @@ sub eth_trunk_status {
     my $ret = ( grep {/Info(.*?)exist/i} @{$rev} ) ? 1 : 0;
 
     #巡检结果命令行输出不存在则跳过
-    $result->{"health"} = "pass";
     return $result if $ret;
 
  #遍历巡检结果以捕捉异常模块信息,仅关注最新的告警信息
     foreach ( @{$rev} ) {
-        if (/init/i) {
+        if (/Unselect/i) {
 
             #将巡检结果命令行分割为字符串数组
             my @info = map {s/(^\s+|\s+$)//r} split( /\:/, $_ );
@@ -885,7 +950,7 @@ sub eth_trunk_status {
             $status =~ s/(^\s+|\s+$)//;
 
             #写入异常状态
-            $result->{"log"} = "eth-trunk is Abnormal"
+            $result->{"log"} = "eth-trunk 部分接口工作异常"
                 if ( $status =~ /not/i );
 
             #设置告警等级
@@ -915,7 +980,9 @@ sub err_down_status {
     my $result;
 
     #初始化巡检结果告警项
-    $result->{"alarm"} = undef;
+    $result->{"alarm"}  = undef;
+    $result->{"health"} = "pass";
+    $result->{"log"}    = undef;
 
     #重组巡检结果拼接为字符串
     my $origin = join( "\n", @{$rev} );
@@ -925,12 +992,13 @@ sub err_down_status {
     my $ret = ( grep {/Info(.*?)exist/i} @{$rev} ) ? 1 : 0;
 
     #巡检结果命令行输出不存在则跳过
-    $result->{"health"} = "pass";
     return $result if $ret;
 
  #遍历巡检结果以捕捉异常模块信息,仅关注最新的告警信息
     foreach ( @{$rev} ) {
-        if (/init/i) {
+        if (/(?<err_down>(efm-threshold-event|efm-remote-failure|bpdu-protection|link-flap|storm-control|port-security|mac-address-flapping|transceiver-power-low|data-integrity-error|runts-error-statistics|auto-defend))/i
+            )
+        {
 
             #将巡检结果命令行分割为字符串数组
             my @info = map {s/(^\s+|\s+$)//r} split( /\:/, $_ );
@@ -942,7 +1010,8 @@ sub err_down_status {
             $status =~ s/(^\s+|\s+$)//;
 
             #写入异常状态
-            $result->{"log"} = "interface in error-down"
+            $result->{"log"}
+                = "部分接口处于 error-down " . "$+{err_down}"
                 if ( $status =~ /not/i );
 
             #设置告警等级
@@ -958,4 +1027,3 @@ sub err_down_status {
 }
 
 1;
-
